/*
 
 Zelda Navi's Quest
 
 Copyright (C) 2013-2014  Vincent Jouillat
 
 Please send bugreports with examples or suggestions to www.zeldaroth.fr
 
 */
#include "AppleInterface.h"

#if defined(NSQ_OSX) || defined(NSQ_IOS)

#if defined(NSQ_OSX)
#  import <Cocoa/Cocoa.h>
#elif defined(NSQ_IOS)
#  import <Foundation/Foundation.h>
#  import <UIKit/UIKit.h>
#endif


/**
 * @brief Return "~/Library/Application Support" or equivalent from the official way, which is available in OSX 10.6+ and iOS 4.0+.
 *
 * Return an OSX 10.0+ and iOS 1.0+ (not compatible with Iphone Simulator) hardcoded equivalent workaround if the 
 * build configuration is set for a lower minimum version.
 * @return The Application Support folder from the User Domain.
 */
std::string get_user_application_support_directory()
{
  @autoreleasepool {

#if defined(NSQ_OSX) && __MAC_OS_X_VERSION_MIN_REQUIRED  >= MAC_OS_X_VERSION_10_6 \
 || defined(NSQ_IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0
    return [[[[[NSFileManager defaultManager]
               URLsForDirectory:NSApplicationSupportDirectory inDomains:NSUserDomainMask]
              objectAtIndex:0]
             path]
            UTF8String];
#else
    return [[NSHomeDirectory() stringByAppendingPathComponent:@"Library/Application Support"]
            UTF8String];
#endif
  }
}

#endif
